*! version 1.0.0  14may1995
program define binom
        version 4.0

        preserve                   /*  Save all the current stata stuff */
        clear                      /*  Clear everything so no conflicts */

        quietly{
            set obs 201
            gen x=.
            gen y=.
            gen x1=.
            gen x2=.
            gen z=.
            gen zero=.}

    global D_sm11 "   Calculating binomial probabilities is difficult when n is large. Fortunately, we can approximate binomial probabilities by areas under a normal curve (which is easy) and this lab illustrates how and when this approximation works."
    window control static D_sm11 5 5 270 25
    
    global D_sm12 "   If we have a binomial random variable with parameters n and pi and the smaller of n*pi and n*(1-pi) is at least five, then binomial probabilities and areas under the normal curve having mean n*pi and variance n*pi*(1-pi) will be almost exactly the same."
    window control static D_sm12 5 35 270 35

    global D_sm13 "   When you choose n and pi and click on Run, the lab will draw a bar graph of the binomial probabilites and superimpose the normal curve. It then shows how the area of one bar (which is a probability) is approximated by the area under the normal curve above that bar."
    window control static D_sm13 5 75 270 40

        * To assign a default value include the following line
        global HJN_V1 = 10
        global HJN_1 "n"
        window control static HJN_1    5 122  5   8
        window control edit     15 122  15  8 HJN_V1

        global HJN_V2 = .5
        global HJN_2 "pi"
        window control static HJN_2   35  122  10  8
        window control edit     50  122 10  8 HJN_V2

        
        window control button "Run"    75 120 25 12 HJN_B2
        window control button "Close"  105 120 25 12 HJN_B3
        window control button "Help"   135 120 25 12 HJN_B4
        
        global HJN_B2 "binomdr"
        global HJN_B4 "whelp binom"
        global HJN_B3 "exit 3000"

    binomdr

        noi window dialog "Normal Approximation to the Binomial Distribution"  5 5 280 150
        restore
end

program define binomdr
        version 4.0

        local n=$HJN_V1

        if `n'< 2 | `n' > 49 {
                sstopbox stop "n must be between 2 and 49"
                        exit}

        local p=$HJN_V2

        if `p' <= 0. | `p' >=1. {
                sstopbox stop "p must be bigger than 0 and less than 1"
                exit}

        local nobs=`n'+1

        gph open
        gph pen 1

        replace x=.
        replace y=.
        replace x = _n-1 in 1/`nobs'
        replace y = exp(lngamma(`n'+1)-lngamma(x+1)-lngamma(`n'-x+1)+x*log(`p') + (`n'-x)*log(1-`p'))        

        local np=int(`n'*`p'+1)
        local yy=y[`np']
        local np1 = `np'-1
        local yy11=y[`np1']

        replace y=`yy'*uniform()

        replace x = -.5 + (`n'+1)*(_n-1)/(_N-1)
        local mu=`n'*`p'
        local var = `n'*`p'*(1-`p')
        replace z = exp(-((x-`mu')^2)/(2*`var'))/sqrt(2*_pi*`var')

        local np1=`n'+1
        graph z x, s(i) xlab(-1,`np1') ylab  l1(" ") l2(" ") b1(" ") b2(" ") /*
            */ bbox(3500,0,23000,32000,850,400,0)
        gphconv

        replace x=.
        replace y=.

        replace x = _n-1 in 1/`nobs'
        replace y = exp(lngamma(`n'+1)-lngamma(x+1)-lngamma(`n'-x+1)+x*log(`p') + (`n'-x)*log(1-`p'))        

        replace x1=x-.5
        replace x2=x+.5
        replace zero=0

        segments x1 zero x1 y
        segments x2 zero x2 y
        segments x1 y x2 y
        segments x1 zero x2 zero

        replace x = -.5 + (`n'+1)*(_n-1)/(_N-1)
        local mu=`n'*`p'
        local var = `n'*`p'*(1-`p')
        replace y=exp(-((x-`mu')^2)/(2*`var'))/sqrt(2*_pi*`var')

        lines x y

        local yy1=-`yy'/10.
        local yy2=1.5*`yy1'
        gph pen 2
        local i=0
        while `i' <= `n' {
            if 2*int(`i'/2)==`i' {drtext `i' `yy1' 0 `i'}
            else {drtext `i' `yy2' 0 `i'}
            local i = `i'+1}

        gph pen 1
        local npx1=`np'-2.5
        local npx2=`np'-1.5
        drbox `npx1' 0 `npx2' `yy11' 

        replace x1=.
        replace x2=.

        replace x1=x if x >= `npx1' & x <= `npx2' 
        replace x2=y if x >= `npx1' & x <= `npx2'
        gph pen 3
        lines x1 x2

        gph text 500 1000 0 -1 Approximate the area of a bar by the area under its part of normal curve
        local npx3=`np'-2
        local npx4=`np'-2
        local npx4= exp(lngamma(`n'+1)-lngamma(`npx4'+1)-lngamma(`n'-`npx4'+1)+`npx4'*log(`p') + (`n'-`npx4')*log(1-`p'))        
        local npx4: display %5.4f `npx4'
        gph text 1500 1500 0 -1 Ex: P(X=`npx3') = Area of bar above `npx3' = `npx4'
        local npx5=(`npx1'-`mu')/sqrt(`var')
        local npx6=(`npx2'-`mu')/sqrt(`var')

        local npx5=normprob(`npx5')
        local npx6=normprob(`npx6')
        local npx7=`npx6'-`npx5'
        local npx5: display %5.4f `npx5'
        local npx6: display %5.4f `npx6'
        local npx7: display %5.4f `npx7'        
        gph text 2500 1500 0 -1 Area under curve between `npx1' and `npx2' is `npx6'-`npx5' = `npx7'


        replace x1=.
        replace x2=.

        replace x1=x if x >= `npx1' & x <= `npx2' & y > `yy11'
        replace x2=y if x >= `npx1' & x <= `npx2' & y > `yy11'
        replace zero=`yy11'
        gph pen 3
        segments x1 zero x1 x2

        replace x1=.
        replace x2=.

        replace x1=x if x >= `npx1' & x <= `npx2' & y < `yy11'
        replace x2=y if x >= `npx1' & x <= `npx2' & y < `yy11'
        replace zero=`yy11'
        gph pen 1
        segments x1 zero x1 x2

        replace x1=.
        replace x2=.

        replace x1=x if x >= `npx1' & x <= `npx2' 
        replace x2=y if x >= `npx1' & x <= `npx2'
        gph pen 3
        lines x1 x2

        gph text 3500 2000 0 -1 Difference = Difference in shaded areas
        gph close

end

exit
