program define poiss
    version 4.0

    preserve                   /*  Save all the current stata stuff */
    clear                      /*  Clear everything so no conflicts */

    quietly{
        set obs 101
        gen x1=.
        gen x11=.
        gen x12=.
        gen x2=.
        gen x21=.
        gen x22=.
        gen y1=.
        gen y2=.
        gen y3=.
        gen zero=.}

    global D_sm11 "   The Poisson Distribution arises in two ways:"
    window control static D_sm11 5 5 290 8

    global D_sm12 "   1) If we have a sequence of events (such as a machine breaking down) and the population of all possible times between occurrences of the events has distribution in the exponential family, then X = # of occurrences in a time interval of length T has the Poisson Distribution. The parameter of the distribution is the average length of time between occurrences of the event. X is discrete and calculating its probabilities is very simple."
    window control static D_sm12 5 15 290 48

    global D_sm13 "   2) The Poisson Distribution can be used to approximate a Binomial Distribution when n is large and pi is small (the parameter of the Poisson is n*pi). This lab illustrates this. You choose values of n and pi and the lab draws bars representing exact binomial probabilities as well as bars using the Poisson Approximation."
    window control static D_sm13 5 65 290 48

    global D_sm14 "Notes: 1) For both distributions, the average value of X is n*pi. 2) Try out various values of n and pi to see when the approximation seems best."
    window control static D_sm14 75 120 200 32

    global D_sm6 "n"
    global D_sm7 "20 50 100 200 300 400 500"
    window control static D_sm6         5 100 10 10
    global D_var1 "50"
    window control ssimple D_sm7 5 110 25 70 D_var1

    global D_sm8 "pi"
    global D_sm9 ".01 .02 .03 .04 .05 .10"
    window control static D_sm8         40 100 20 10
    global D_var2 ".10"
    window control ssimple D_sm9 40 110 25 65 D_var2

    window control button "Run"     115 160 30 14 D_b1
    window control button "Close"   150 160 30 14 D_b2
    window control button "Help"    185 160 30 14 D_b3 

    global D_b1 "poissdr"
    global D_b2 "exit 3000"
    global D_b3 "whelp poiss"

    poissdr

    cap noi window dialog "The Poisson Distribution" $D_dlgx $D_dlgy 300 200

    restore
end

program define poissdr
    version 4.0

    gph open
    gph pen 1

    local n      = $D_var1
    local pi     = $D_var2
    local lambda = `n' * `pi'
    local var    = `lambda' * (1-`pi')
    local xmax   = max(10,int(`lambda' + 3 * sqrt(`var')))

*    gph text 1000 16000 0 0 `n' `pi' `lambda' `var' `xmax'

    replace x1=.
    replace x2=.
    replace y1=.
    replace y2=.
    replace y3=.

    local nobs = `xmax'+1

*    gph text 2000 16000 0 0 `N' `D' `n' `p' `nobs'

    replace x1 = _n-1 in 1/`nobs'
    replace y1 = exp(lngamma(`n'+1)-lngamma(x1+1)-lngamma(`n'-x1+1)+x1*log(`pi') + (`n'-x1)*log(1-`pi'))

    summarize y1
    local y1max = _result(6)
*    gph text 3000 16000 0 0 `y1max'
    
    replace y2 = exp(-`lambda' + x1*log(`lambda') - lngamma(x1+1))

    summarize y2
    local y2max = _result(6)

*    gph text 4000 16000 0 0 `y2max'

    local ymax = max(`y1max',`y2max')

*    gph text 5000 16000 0 0 `y1max' `y2max' `ymax'

    graph y1 x1, s(i) xlab(-1,`nobs') ylab  l1(" ") l2(" ") b1(" ") b2(" ") /*
            */ bbox(3500,0,23000,32000,850,400,0)
    gphconv

    local yy1=-`ymax'/10.
    local yy2=1.5*`yy1'
    gph pen 2
    local i=0
    while `i' <= `xmax' {
        if 2*int(`i'/2)==`i' {drtext `i' `yy1' 0 `i'}
        else {drtext `i' `yy2' 0 `i'}
        local i = `i'+1}

    replace x11=x1-.5
    replace x12=x1+.5
    replace zero=0

    gph pen 1
    segments x11 zero x11 y1
    segments x12 zero x12 y1
    segments x11 y1 x12 y1
    segments x11 zero x12 zero


    replace x21=x1-.25
    replace x22=x1+.25
    replace zero=0

    gph pen 3
    segments x21 zero x21 y2
    segments x22 zero x22 y2
    segments x21 y2 x22 y2
    segments x21 zero x22 zero

    gph pen 2

    gph text 1000 16000 0 0 Poisson approximation (thin bars) to binomial (wide bars) probabilities
    gph text 2000 16000 0 0 n=`n', pi=`pi'

    gph close

end
exit
